;(function(window, $) {
	'use strict';

    $.fn.selectRange = function(start, end) {
        if (end === undefined) {
            end = start;
        } 
        return this.each(function() {
            if ('selectionStart' in this) {
                this.selectionStart = start;
                this.selectionEnd = end;
            } else if (this.setSelectionRange) {
                this.setSelectionRange(start, end);
            } else if (this.createTextRange) {
                var range = this.createTextRange();
                range.collapse(true);
                range.moveEnd('character', end);
                range.moveStart('character', start);
                range.select();
            }
        });
    };

    $.fn.cypDialog = function( options ) {
        var settings = $.extend({
            title: 'Clever YouTube Plugin',
            autoOpen: false,
            closeText: '',
            dialogClass: 'cyp-dialog cyp-dialog-confirmation',
            modal: true,
            resizable: false,
            draggable: false,
            buttons: {
                'OK': function() {
                    $(this).dialog('close');
                }
            },
        }, options );
        
        return this.dialog(settings);
    };

    window.cyp = {};
    window.cyp.audio = {};
    window.cyp.audio.play = function (path) {
        var audio = new Audio(WPURLS.cypurl + '/' + path);
        audio.play();
        return audio;
    };
    window.cyp.audio.load = function (path) {
        var audio = new Audio(WPURLS.cypurl + '/' + path);
        audio.preload = 'auto';
        return audio;
    };
    window.cyp.youtube = {};
    window.cyp.youtube.parseUrl = function (url) {
        var youTubeIdLength = 11;

        if (url && typeof url === 'string') {
            var tests = [
                parseByParam,
                parseByShortUrl,
                parseByEmbed,
                parseById
            ];

            for (var i = 0; i < tests.length; i++) {
                var result = tests[i](url);

                if (result) {
                    return result;
                }
            }
        }
        
        return false;

        function parseByParam(url) {
            return parseByNeedle(url, 'v=');
        }
        function parseByShortUrl(url) {
            return parseByNeedle(url, 'youtu.be/');
        }
        function parseByEmbed(url) {
            return parseByNeedle(url, 'youtube.com/embed/');
        }
        function parseById(url) {
            if (url.length === youTubeIdLength) {
                return url;
            } else {
                return false;
            }
        }
        function parseByNeedle(url, needle) {
            var index = url.indexOf(needle);

            if (index > -1) {
                var start = index + needle.length;
                var end = start + youTubeIdLength;
                var str = url.substring(start, end);

                if (str.length === youTubeIdLength) {
                    return str;
                }
            }
            
            return false;
        }
    };

})(window, jQuery);